#!/usr/bin/perl -w

# $Id: dr_choose-server.pl,v 1.1 2006/08/06 22:31:44 semik Exp $
#
# dr_choose-server.pl is plugin to implementing dead-realm feature for
# Radiator as replacement for dead-host feature which is not suitable
# for proxy networks such as eduroam.
#
# Author: Jan Tomek <jan.tomasek@cesnet.cz>, <jan@tomasek.cz>
# Copyright (c) 2006, CESNET, z.s.p.o.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# See README and COPYING for more details.

sub {
  my @args = @_;

  # Get parameters from radiator config
  my $timeout = &main::getVariable('dr_mark_timeout') || 60*60;
  my $identifier = $args[0]->{Handler}->{Identifier};
  my $servers = &main::getVariable('dr_'.$identifier.'_server_list');
  my @servers = split(' *, *', $servers);
  unless (@servers) {
    &main::log($main::LOG_DEBUG,
	       "dead-realm: There are not defined servers for Handler Identifier=$identifier.");
    return undef;
  };

  my $username = $args[0]->get_attr('User-Name');
  my $realm = semik_get_realm($username) || 'unknown-realm';

  my %servers;

  # At first we try to find server which is has not marked realm we
  # are processing now as dead, or server which have that realm marked
  # as for too long.
  #
  # If you wish to stay with one server until it goes down set set
  # $dr_mark_timeout unbelivable big value. Such as one year
  # 365*24*60*60.
  foreach my $server (@servers) {
    if (defined($main::deadRealms->{$server}->{$realm})) {
      # Check if this realm is not makred as dead for too long.
      if (time - $main::deadRealms->{$server}->{$realm} > $timeout) {
	undef $main::deadRealms->{$server}->{$realm};
	&main::log($main::LOG_DEBUG,
		   "dead-realm: Clearing too old dead mark for \@$realm on host $server.");
      } else {
	# Queue this server for later if we will not find any better
	$servers{$server} = $main::deadRealms->{$server}->{$realm};
	next;
      };
    };

    my $auth = Radius::AuthGeneric::find($server);
    if ($auth) {
      return $auth->handle_request(@args);
    } else {
      &main::log($main::LOG_ERROR, "dead-realm: Can't locate AuthBy handler $server");
      return undef;
    };
  };

  # If we failed to find server which has not dead mark for realm we
  # are processing, we do not give up! We will try to send our packet
  # to server with oldest dead-realm mark. Chosing server with oldest
  # dead-realm mark cause that, we will cycle throught list untill
  # some of them will become alive.
  #
  # If server is dead, than it's dead-realm mark is updated by
  # NoReplyHook in it's Auth block.
  #
  # If server is alive, than it's dead-realm mark is removed at moment
  # we receive some response from from it. That is done by ReplyHook
  # in it's Auth block.
  foreach my $server (sort {$servers{$a} <=> $servers{$b}} keys %servers) {
    my $auth = Radius::AuthGeneric::find($server);
    return $auth->handle_request(@args) if ($auth);
  };

  # We are loosers there is not any valid AuthBy ... That is probably
  # missconfiguration.
  &main::log($main::LOG_ERROR, "dead-realm: Can't locate ANY defined AuthBy");
  return undef;
};
